#!/usr/bin/env bash

set -e

trap "echo 'terminating generate-backups script'" EXIT

MOUNT=$1
LOGS=$2
SCALE=$3


DATADIR=$MOUNT/test-$LOGS/data
DEBUG=$LOGS/debug.log
BACKUPS=$MOUNT/test-$LOGS/backups
CHECKSUMS=$MOUNT/test-$LOGS/checksums

# backup index
backup=0

killall -9 postgres || true
sleep 5


echo `date` "0/0" GENERATOR initializing cluster

# init and start the cluster
rm -Rf $DATADIR $BACKUPS $CHECKSUMS $LOGS/pg.log $DEBUG

pg_ctl -D $DATADIR -o "-k" init >> $DEBUG 2>&1
#pg_ctl -D $DATADIR init >> $DEBUG 2>&1
cp postgresql.conf $DATADIR

# randomize WAL size a little bit, to change how often we trigger checkpoints
mws=$((1 + RANDOM % 8))

echo "max_wal_size = ${mws}GB" >> $DATADIR/postgresql.conf


mkdir $BACKUPS $CHECKSUMS

# maybe enable/disable checksums?

# start the cluster
pg_ctl -D $DATADIR -l $LOGS/pg.log start >> $DEBUG 2>&1

# initialize the database
echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "creating pgbench databases (db, scale $SCALE)"
createdb db

echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "creating amcheck extension"
psql db -c "create extension amcheck" >> $LOGS/debug.log 2>&1

echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "pgbench init (db)"
pgbench -i -s $SCALE db >> $LOGS/debug.log 2>&1
echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "pgbench database db initialized (db)"

# forger previous runs
rm -f "workload.done"

# start the workload that creates/drops databases and runs pgbench on them
./generate-workload.sh $MOUNT $LOGS $SCALE 15 &

# now run pgbench on all the databases, with more and more changes
while /bin/true; do

	# stop after this run, if there's she .stop file"
	stop=0
	if [ -f "workload.done" ]; then
		stop=1
	fi

	backup=$((backup+1))

	# generate another backup
	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup $backup : performing base backup"

	# checkpoint mode
	r=$((RANDOM % 2))

	# random strategy to create the database
	if [ "$r" == 0 ]; then
		checkpoint="fast"
	else
		checkpoint="spread"
	fi

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup $backup START : pg_basebackup -c $checkpoint -D $BACKUPS/backup"

	# create another backup
	pg_basebackup -c $checkpoint --manifest-checksums=sha256 -D $BACKUPS/backup >> $DEBUG 2>&1

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup $backup STOP : backup completed"

	tar -cf $BACKUPS/backup-$backup.tar $BACKUPS/backup > /dev/null 2>&1
	gzip -1 $BACKUPS/backup-$backup.tar

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup $backup STOP : verifying backup"

	./verify-backup.sh $BACKUPS/backup $LOGS
	res=$?

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup verification result: $res"

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR "backup $backup STOP : backup verified"

	rm -Rf $BACKUPS/backup

	# maybe stop (if workload completed)
	if [ "$stop" == "1" ]; then
		break
	fi

	# sleep for a bit between runs
	sleep $((5 + RANDOM % 30))

done

echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") GENERATOR stopping cluster
pg_ctl -D $DATADIR stop >> $LOGS/debug.log 2>&1
echo `date` "0/0" "cluster stopped"

rm -Rf $DATADIR

killall -9 postgres || true

echo "$LOGS : $backup backups generated"
