#!/usr/bin/env bash

set -e

trap "echo 'terminating generate-workload script'" EXIT

MOUNT=$1
LOGS=$2
SCALE=$3
LOOPS=$4

DATADIR=$MOUNT/test-$LOGS/data
DEBUG=$LOGS/debug.log
BACKUPS=$MOUNT/test-$LOGS/backups
CHECKSUMS=$MOUNT/test-$LOGS/checksums


MAXPCT=10

echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD loops $LOOPS maxpct $MAXPCT


# now run pgbench on all the databases, with more and more changes
for l in $(seq 1 $LOOPS); do

	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "loop $l of $LOOPS"

	pct=$((1 + RANDOM % MAXPCT))

	# scale is a multiple of 100k rows in pgbench_accounts, but only ~1640 pages
	# and the backups work at page level, so assume each update hits one random page
	# (ignore collisions)
	ROWS=$((1640 * SCALE * pct / 100))

	# calculate number of clients and transactions per client
	c=$((16))
	t=$((ROWS/c))

	# drop the database copy (if exists)
	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "dropping database db_copy"
	./drop-db.sh
	echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "db_copy dropped"

	# now do three things almost at the same time - create a copy of the database, start pgbench on both the source and the new DB
	r=$((RANDOM % 2))

	# random strategy to create the database
	if [ "$r" == 0 ]; then
		strategy="wal_log"
	else
		strategy="file_copy"
	fi

	# start the CREATEDB from the source db
	./create-db.sh $strategy &

	# give the createdb bit of time to start
	sleep 1

	# start a pgbench on the first database
	./pgbench.sh $LOGS $t $c db &

        # maybe do some bulk cleanup operations
        r=$((RANDOM % 3))

        if [ "$r" == "0" ]; then
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum db / start"
                psql db -c "vacuum" > /dev/null 2>&1
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum db / done"
        elif [ "$r" == "1" ]; then
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum full db / start"
                psql db -c "vacuum full" > /dev/null 2>&1
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum full db / done"
        fi

	# wait for the createdb and pgbench runs to complete
	wait

	# now also run pgbench on the new database
	./pgbench.sh $LOGS $t $c db_copy &

        # maybe do some bulk cleanup operations on the copy db too
        r=$((RANDOM % 3))

        if [ "$r" == "0" ]; then
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum db_copy / start"
                psql db_copy -c "vacuum" > /dev/null 2>&1 
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum db_copy / done"
        elif [ "$r" == "1" ]; then
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum full db_copy / start"
                psql db_copy -c "vacuum full" > /dev/null 2>&1
                echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "vacuum full db_copy / done"
        fi

        # wait for the second pgbench to complete
        wait

	# sleep for a bit 5-35 seconds
	sleep $((5 + RANDOM % 30))

done

touch workload.done

echo `date` $(psql -t -A postgres -c "SELECT pg_current_wal_lsn()") WORKLOAD "workload generator completed"
