#!/usr/bin/env bash

set -e

BACKUP=$1
LOGS=$2

DEBUG=debug.log
RUNS=1

t=$(date +%s)

LOGDIR=$LOGS/verify-$t

mkdir -p $LOGDIR


# change port, to start next to original cluster
PGPORT=6000
echo "port = 6000" >> $BACKUP/postgresql.conf

# make sure no other cluster is running
pg_ctl -D $BACKUP -m immediate stop > /dev/null 2>&1 || true

# restart the cluster, so finish recovery before checksums check
pg_ctl -D $BACKUP -l $LOGDIR/pg.restore.log start > /dev/null 2>&1

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "redo done"
lsn=$(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()")
pg_ctl -D $BACKUP -l $LOGDIR/pg.restore.log stop > /dev/null 2>&1

sync

echo `date` $lsn VERIFIER "recovery finished, verifying checksums"

# verify checksums
pg_checksums --check $BACKUP >> $LOGDIR/checksums.log 2>&1
chsums=$?

echo `date` $lsn VERIFIER "checksums verified ($chsums)"

# start the cluster again
pg_ctl -D $BACKUP -l $LOGDIR/pg.restore.log start > /dev/null 2>&1

# drop the db_copy database, if it's invalid thanks to dropdb
c=$(psql -p 6000 postgres -t -A -c "select (datconnlimit = -2) from pg_database where datname = 'db_copy'")
if [ "$c" == "t" ]; then
	echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIE "dropping invalid database db_copy"
	dropdb -p 6000 --if-exists db_copy
fi

# amcheck

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "running amcheck (db)"

psql -p 6000 db < check-tables.sql >> $LOGDIR/amcheck-tables-check.log
psql -p 6000 db < check-indexes.sql >> $LOGDIR/amcheck-indexes-check.log

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "amcheck completed (db)"

# SQL check

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "running SQL check (db)"

psql -p 6000 db -c "SELECT 'accounts', count(*), sum(abalance) FROM pgbench_accounts" >> $LOGDIR/sql-check.log
psql -p 6000 db -c "SELECT 'branches', count(*), sum(bbalance) FROM pgbench_branches" >> $LOGDIR/sql-check.log
psql -p 6000 db -c "SELECT 'tellers', count(*), sum(tbalance) FROM pgbench_tellers" >> $LOGDIR/sql-check.log
psql -p 6000 db -c "SELECT 'history', count(*), sum(delta) FROM pgbench_history" >> $LOGDIR/sql-check.log

s1=$(psql -p 6000 db -t -A -c "SELECT sum(abalance) FROM pgbench_accounts")
s2=$(psql -p 6000 db -t -A -c "SELECT sum(bbalance) FROM pgbench_branches")
s3=$(psql -p 6000 db -t -A -c "SELECT sum(tbalance) FROM pgbench_tellers")

# history may be empty, make sure to return 0 in that case
s4=$(psql -p 6000 db -t -A -c "SELECT coalesce(sum(delta),0) FROM pgbench_history")

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "SQL check completed (db) (account $s1 branches $s2 tellers $s3 history $s4)"

if [ "$s1" != "$s2" ] || [ "$s1" != "$s3" ] || [ "$s1" != "$s4" ]; then
	echo "ERROR: balance mismatch (db) $p $b $method $manifest $checksums"
	result="ERROR"
fi

# does the db_copy DB exist?
x=$(psql -p 6000 -t -A -c "select 1 from pg_database where datname = 'db_copy'" db)

if [ "x$x" == "x1" ]; then

	echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "running amcheck (db_copy)"

	psql -p 6000 db_copy < check-tables.sql >> $LOGDIR/amcheck-tables-check.log
	psql -p 6000 db_copy < check-indexes.sql >> $LOGDIR/amcheck-indexes-check.log

	echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "amcheck completed (db_copy)"

	echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "running SQL check (db_copy)"

	psql -p 6000 db_copy -c "SELECT 'accounts', count(*), sum(abalance) FROM pgbench_accounts" >> $LOGDIR/sql-check.log
	psql -p 6000 db_copy -c "SELECT 'branches', count(*), sum(bbalance) FROM pgbench_branches" >> $LOGDIR/sql-check.log
	psql -p 6000 db_copy -c "SELECT 'tellers', count(*), sum(tbalance) FROM pgbench_tellers" >> $LOGDIR/sql-check.log
	psql -p 6000 db_copy -c "SELECT 'history', count(*), sum(delta) FROM pgbench_history" >> $LOGDIR/sql-check.log

	s1=$(psql -p 6000 db_copy -t -A -c "SELECT sum(abalance) FROM pgbench_accounts")
	s2=$(psql -p 6000 db_copy -t -A -c "SELECT sum(bbalance) FROM pgbench_branches")
	s3=$(psql -p 6000 db_copy -t -A -c "SELECT sum(tbalance) FROM pgbench_tellers")

	# history may be empty, make sure to return 0 in that case
	s4=$(psql -p 6000 db_copy -t -A -c "SELECT coalesce(sum(delta),0) FROM pgbench_history")

	echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "SQL check completed (db_copy) (account $s1 branches $s2 tellers $s3 history $s4)"

	if [ "$s1" != "$s2" ] || [ "$s1" != "$s3" ] || [ "$s1" != "$s4" ]; then
		echo "ERROR: balance mismatch (db_copy) $p $b $method $manifest $checksums"
		result="ERROR"
	fi

fi

echo `date` $(psql -p 6000 -t -A postgres -c "SELECT pg_current_wal_lsn()") VERIFIER "done: " $((used_after - used_before)) "KB, $d seconds, checksums $chsums md5 $chsum_data verify $verify"

pg_ctl -D $BACKUP -l $LOGDIR/pg.restore.log stop > /dev/null 2>&1

echo `date` $lsn $(du -s $BACKUP $BACKUP/base)

ls $BACKUP/base >> $LOGDIR/dbs-check.log
find $BACKUP >> $LOGDIR/files-check.log

if [ "$result" == "ERROR" ]; then
	exit 1
fi

exit 0
